#include "driver.h"
#include "mcu_driver.h"

#define TRIGGER_MASK                0x1F
#define TRIGGER_SHIFT               5

#define ID_SHIFT                    4
#define ID_MASK                     0x03

#define TYPE_SHIFT                  4
#define TYPE_MASK                   0x03

#define HWV_MASK                    0x0F

#define SIGNED_MASK                 0x80
#define HALF_BYTE_SHIFT             4
#define SIGNED_INT16_MASK           0xF000
#define TEMP_LSB_MASK               0x0C
#define MAGNETO_LSB_MASK            0x0F
#define UNSIGNED_MASK               0x7F
#define SIGNED_VAL                  -2048
#define UNSGINED_VAL                0

#define TEMPERATURE_OFFSET           1180
#define TEMPERATURE_OFFSET_VAL       25
#define TEMPERATURE_RES              0.24

void magneto6_generic_write ( uint8_t reg_adr, uint8_t trigger, uint8_t tx_data )
{
    uint8_t temp_data;

    temp_data = reg_adr & TRIGGER_MASK;
    temp_data |= trigger << TRIGGER_SHIFT;

    dev_i2c_start(  );
    dev_i2c_write( MAGNETO6_SLAVE_ADR_W );
    dev_i2c_write( temp_data );
    dev_i2c_write( tx_data );
    dev_i2c_stop(  );
}

uint8_t magneto6_generic_read ( uint8_t reg_adr, uint8_t trigger )
{
    uint8_t temp_data;

    temp_data = reg_adr & TRIGGER_MASK;
    temp_data |= trigger << TRIGGER_SHIFT;

    dev_i2c_start(  );
    dev_i2c_write( MAGNETO6_SLAVE_ADR_R );
    dev_i2c_write( temp_data );
    temp_data = dev_i2c_read(  );
    dev_i2c_stop(  );

    return temp_data;
}

uint8_t magneto6_get_id ( void )
{
    uint8_t id;
    id = magneto6_generic_read( MAGNETO6_REG_TEMP_3_2_ID_AXIS_Z_3_0, 
                                MAGNETO6_TRIGGER_NO );
    id >>= ID_SHIFT;
    id &= ID_MASK;
    return id;
}

uint8_t magneto6_get_type ( void )
{
    uint8_t dev_type;
    dev_type = magneto6_generic_read( MAGNETO6_REG_VERSION, 
                                      MAGNETO6_TRIGGER_NO );
    dev_type >>= TYPE_SHIFT;
    dev_type &= TYPE_MASK;
    return dev_type;
}

uint8_t magneto6_get_hw_ver ( void )
{
    uint8_t hw_ver;
    hw_ver = magneto6_generic_read( MAGNETO6_REG_VERSION, 
                                    MAGNETO6_TRIGGER_NO );
    hw_ver &= HWV_MASK;
    return hw_ver;
}

int16_t magneto6_get_temp_value ( void )
{
    int16_t temperature_value = 0;
    uint8_t temp_read;

    temp_read = magneto6_generic_read( MAGNETO6_REG_TEMP_11_4, 
                                       MAGNETO6_TRIGGER_NO );
    temperature_value = temp_read;
    temperature_value <<= HALF_BYTE_SHIFT;
    if ( 0 != ( temp_read  & SIGNED_MASK ) )
    {
        temperature_value |= SIGNED_INT16_MASK;
    }
    temp_read = magneto6_generic_read( MAGNETO6_REG_TEMP_3_2_ID_AXIS_Z_3_0, 
                                       MAGNETO6_TRIGGER_NO );
    temp_read >>= HALF_BYTE_SHIFT;
    temp_read &= TEMP_LSB_MASK;
    temperature_value |= temp_read;

    temperature_value -= TEMPERATURE_OFFSET;
    temperature_value *= TEMPERATURE_RES;
    temperature_value += TEMPERATURE_OFFSET_VAL;

    return temperature_value;
}

int16_t magneto6_get_axis_value ( uint8_t axis )
{
    int16_t axis_value;
    uint8_t msb_read;
    uint8_t lsb_read;
    int16_t signed_val = SIGNED_VAL;

    switch ( axis )
    {
        case MAGNETO6_AXIS_X:
        {
            msb_read = magneto6_generic_read( MAGNETO6_REG_AXIS_X_11_4, 
                                              MAGNETO6_TRIGGER_NO );
            lsb_read = magneto6_generic_read( MAGNETO6_REG_AXES_XY_3_0, 
                                              MAGNETO6_TRIGGER_NO );
            lsb_read >= HALF_BYTE_SHIFT;
            break;
        }
        case MAGNETO6_AXIS_Y:
        {
            msb_read = magneto6_generic_read( MAGNETO6_REG_AXIS_Y_11_4, 
                                              MAGNETO6_TRIGGER_NO );
            lsb_read = magneto6_generic_read( MAGNETO6_REG_AXES_XY_3_0, 
                                              MAGNETO6_TRIGGER_NO );
            break;
        }
        case MAGNETO6_AXIS_Z:
        {
            msb_read = magneto6_generic_read( MAGNETO6_REG_AXIS_Z_11_4, 
                                              MAGNETO6_TRIGGER_NO );
            lsb_read = magneto6_generic_read( MAGNETO6_REG_TEMP_3_2_ID_AXIS_Z_3_0, 
                                              MAGNETO6_TRIGGER_NO );
            break;
        }
        default:
        {
            return MAGNETO6_DEV_ERROR;
        }
    }

    if ( 0 == ( msb_read  & SIGNED_MASK ) )
    {
        signed_val = UNSGINED_VAL;
    }
    msb_read &= UNSIGNED_MASK;
    lsb_read &= MAGNETO_LSB_MASK;
    axis_value = msb_read;
    axis_value <<= HALF_BYTE_SHIFT;
    axis_value |= lsb_read;
    axis_value += signed_val;
    return axis_value;
}

void magneto6_get_axes_value ( magneto6_axes_t *axes )
{
    axes->x = magneto6_get_axis_value( MAGNETO6_AXIS_X );
    axes->y = magneto6_get_axis_value( MAGNETO6_AXIS_Y );
    axes->z = magneto6_get_axis_value( MAGNETO6_AXIS_Z );
}