#ifndef DRIVER
#define DRIVER

#include "stdint.h"

/**
 * @brief slave address
 */
#define MAGNETO6_SLAVE_ADR                    0x35
#define MAGNETO6_SLAVE_ADR_W                  0x6A
#define MAGNETO6_SLAVE_ADR_R                  0x6B

/**
 * @brief register addresses
 */
#define MAGNETO6_REG_AXIS_X_11_4              0x00
#define MAGNETO6_REG_AXIS_Y_11_4              0x01
#define MAGNETO6_REG_AXIS_Z_11_4              0x02
#define MAGNETO6_REG_TEMP_11_4                0x03
#define MAGNETO6_REG_AXES_XY_3_0              0x04
#define MAGNETO6_REG_TEMP_3_2_ID_AXIS_Z_3_0   0x05
#define MAGNETO6_REG_STATUS                   0x06
#define MAGNETO6_REG_CFG                      0x10
#define MAGNETO6_REG_CTRL_1                   0x11
#define MAGNETO6_REG_CTRL_2                   0x13
#define MAGNETO6_REG_VERSION                  0x16

/**
 * @brief trigger bits set
 */
#define MAGNETO6_TRIGGER_NO                   0x00
#define MAGNETO6_TRIGGER_ADC_FINISHED         0x03

/**
 * @brief configuration and control values
 */
#define MAGNETO6_CFG_ID                       0xC0
#define MAGNETO6_CFG_TEMPEARATURE             0x40
#define MAGNETO6_CFG_MAGNETO                  0x80
#define MAGNETO6_CTRL1_INT_DISABLE            0x04

/**
 * @brief axes macro
 */
#define MAGNETO6_AXIS_X                       0xAA
#define MAGNETO6_AXIS_Y                       0xBB
#define MAGNETO6_AXIS_Z                       0xCC

/**
 * @brief device error
 */
#define MAGNETO6_DEV_ERROR                    0xFFFF            

/**
 * @brief structure for storing axes values
 */
typedef struct
{
    int16_t x;
    int16_t y;
    int16_t z;
}magneto6_axes_t;

/**
 * @brief Generic function for writing data to register
 * 
 * @param[in]    reg_adr         register address
 * 
 * @param[in]    trigger         trigger bits config
 * 
 * @param[in]    tx_data         data to be written
 */
void magneto6_generic_write ( uint8_t reg_adr, uint8_t trigger, uint8_t tx_data );

/**
 * @brief Generic function for reading data from registar
 * 
 * @param[in]    reg_adr         register address
 * 
 * @param[in]    trigger         trigger bits config
 * 
 * @return rx data
 */
uint8_t magneto6_generic_read ( uint8_t reg_adr, uint8_t trigger );

/**
 * @brief Function for getting hw version of device
 * 
 * @return HW version of the device
 */
uint8_t magneto6_get_hw_ver ( void );

/**
 * @brief Function for getting device type
 * 
 * @return Type of the device
 */
uint8_t magneto6_get_type ( void );

/**
 * @brief Function for getting id device
 * 
 * @return ID of the device
 */
uint8_t magneto6_get_id ( void );

/**
 * @brief Function for getting device temperature value
 * 
 * @return temperature of the device
 */
int16_t magneto6_get_temp_value ( void );

/**
 * @brief Function for getting device axis magneto value
 * 
 * @param[in]    axis     axis macro
 * 
 * @return axis of the device or error value for wrong macro
 */
int16_t magneto6_get_axis_value ( uint8_t axis );
/**
 * @brief Function for getting device axes magneto values
 * 
 * @param[out]   axes     structure of all 3 axes values
 */
void magneto6_get_axes_value ( magneto6_axes_t *axes );

#endif